import re

RE_PARAMS = re.compile('(/:([a-zA-Z_]+))')


def to_url(urltemplate, params, strict=False):
    uri = urltemplate

    params_to_replace = RE_PARAMS.findall(urltemplate)

    if params_to_replace:
        for needle, key in params_to_replace:
            try:
                uri = uri.replace(needle, '/%s' % params[key])
            except KeyError:
                if strict:
                    raise
    return uri


def remove_parameters(urltemplate):
    uri = urltemplate

    params_to_replace = RE_PARAMS.findall(urltemplate)
    for needle, key in params_to_replace:
        uri = uri.replace(needle, '')
    return uri


def to_django_urlpattern(path):
    return RE_PARAMS.sub('/(?P<\\2>[^/]+)', path)





import json
import datetime
import decimal


__all__ = ['JsonSerializer', 'registry']



class DefaultRestfulEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.isoformat()
        elif isinstance(obj, datetime.date):
            return obj.isoformat()
        elif isinstance(obj, datetime.timedelta):
            return (datetime.datetime.min + obj).time().isoformat()
        elif isinstance(obj, decimal.Decimal):
            return float(str(obj))
        else:
            return super(DefaultRestfulEncoder, self).default(obj)


class DateTimeJsonSerializer(object):
    def dumps(self, obj):
        return json.dumps(obj, cls=DefaultRestfulEncoder)

    def loads(self, txt):
        obj = json.loads(txt)
        return obj


class JsonSerializer(object):
    def __init__(self):
        self._json = DateTimeJsonSerializer()

    def loads(self, ctx):
        return self._json.loads(ctx.raw)

    def dumps(self, data):
        return self._json.dumps(data)


class MultiPartFormDataSerializer(object):
    def loads(self, ctx):
        from django.utils.datastructures import MultiValueDict
        data = MultiValueDict()
        data.update(ctx.data)
        data.update(ctx.files)
        return data

    def dumps(self, data):
        raise NotImplementedError


class AlreadyRegistered(Exception):
    pass


class SerializersRegistry(object):
    def __init__(self):
        self._serializers = {}

    def _key(self, mimetype):
        return mimetype.lower()

    def __getitem__(self, item):
        return self._serializers[self._key(str(item))]

    def register(self, mimetype, instance):
        key = self._key(mimetype)
        if key in self._serializers:
            raise AlreadyRegistered(key)
        self._serializers[key] = instance

    def mimetypes(self):
        return self._serializers.keys()

    def items(self):
        return self._serializers.items()

    def contains(self, mimetype):
        return self._key(mimetype) in self._serializers


default_serializers = SerializersRegistry()
default_serializers.register('application/json', JsonSerializer())
default_serializers.register('multipart/form-data', MultiPartFormDataSerializer())


def register(mimetype, serializer):
    default_serializers.register(mimetype, serializer)


def get(mimetype):
    return default_serializers.get(mimetype)





import functools
import logging
import mimeparse
import responses
import urltemplate
import urllib
import sys

from collections import OrderedDict
from django.http import HttpResponse
from django.conf import settings

from .serializers import default_serializers
from .headers import (
        normalize_header_name,
        parse_accept_header,
        build_content_type_header,
        )
from .context import Context
from .exceptions import Http404


log = logging.getLogger(__name__)


DEFAULT_REPRESENTATION_KEY = '__default__'


def http_response(response):
    """
    RESTResponse -> HTTPResponse factory
    """

    if isinstance(response, HttpResponse):
        return response

    context = response.context

    if response.data is not None:
        content_type = context.response_content_type
        serializer = context.serializer
        representation = context.representation_name
        content = serializer.dumps(response.serialize(response.data, representation))
    else:
        content = ''
        content_type = 'application/json'

    httpresp = HttpResponse(content, status=response.status)

    if content_type:
        httpresp['Content-Type'] = content_type

    for header, value in response.headers.items():
        httpresp[header]=value

    return httpresp


def resource_name_from_path(path):
    return urltemplate.remove_parameters(path).strip('/')


class Resource(object):
    def __init__(self, path, name=None, expose=False, serializers=None):
        self._path = path
        self._callbacks = {}
        self._expose = expose
        self._links = {}
        self._name = name or resource_name_from_path(path)
        self._representations = OrderedDict()
        self._serializers = serializers or default_serializers
        # register aliases for the decorators
        for verb in ('GET', 'POST', 'PUT', 'DELETE', 'OPTIONS', 'PATCH'):
            setattr(self, verb.lower(), functools.partial(self._decorator, verb))

    def _decorator(self, method, link_to=None, link_as=None):
        def wrapper(view):
            if method in self._callbacks:
                raise ValueError('Already registered')
            self._callbacks[method] = view
            if link_to:
                key = link_as or link_to.__name__
                link_to._links[key] = (method, self)
            return view
        return wrapper

    @property
    def name(self):
        return self._name

    @property
    def expose(self):
        return self._expose

    @property
    def path(self):
        return self._path

    @property
    def serializers(self):
        return self._serializers

    @property
    def representations(self):
        return self._representations

    def __call__(self, ctx, *args, **kw):
        from django.http import Http404 as DjangoHttp404

        method = ctx.method
        request = ctx.request

        try:
            content_length = int(request.META['CONTENT_LENGTH'])
        except (KeyError, TypeError, ValueError):
            content_length = 0

        if content_length and 'CONTENT_TYPE' in request.META:
            mimetype = mimeparse.best_match(dict(self._serializers.items()),
                    request.META['CONTENT_TYPE'])
            if mimetype:
                ctx.deserializer = self._serializers[mimetype]
                if request.body:
                    ctx.body = self._serializers[mimetype].loads(ctx)
            elif not content_length:
                self.body = None
            else:
                return http_response(ctx.NotAcceptable())

        ctx.content_type = request.META.get('CONTENT_TYPE')

        # prepare request headers

        headers = request.META.items()
        http_headers = dict(map(lambda x: (normalize_header_name(x[0]),x[1]),
            filter(lambda x: x[0].startswith('HTTP_'), headers)))
        ctx.headers.update(http_headers)

        # match response representation, serializer and content type

        response_content_type = None
        response_serializer = None
        response_representation = None

        if 'accept' in ctx.headers:
            try:
                accepting = parse_accept_header(ctx.headers['accept'])
            except ValueError:
                content_type = None
            else:
                for content_type, representation, q in accepting:
                    if content_type == '*/*' or content_type == 'application/*':
                        content_type = 'application/json'
                        response_representation = DEFAULT_REPRESENTATION_KEY
                    if ctx.resource.serializers.contains(content_type)\
                            and (not representation or representation in ctx.resource.representations):
                        response_representation = representation or DEFAULT_REPRESENTATION_KEY
                        response_serializer = ctx.resource.serializers[content_type]
                        content_type = build_content_type_header(content_type, representation)
                        break

                if content_type and not response_serializer:
                    try:
                        response_serializer = ctx.resource.serializers[content_type]
                    except KeyError:
                        return http_response(ctx.NotAcceptable())

        else:
            content_type = 'application/json'
            response_serializer = ctx.resource.serializers[content_type]
            response_representation = DEFAULT_REPRESENTATION_KEY

        response_content_type = content_type

        if content_length and (not response_content_type or not response_serializer):
            return HttpResponse('Not acceptable `%s`' % ctx.headers.get('accept'),
                    status=406) # Not Acceptable

        ctx.representation_name = response_representation
        ctx.response_content_type = response_content_type
        ctx.serializer = response_serializer


        # support for X-HTTP-METHOD-OVERRIDE
        method = http_headers.get('x-http-method-override') or method

        log.debug('Calling %s, %s, %s' % (method, args, kw))
        if method in self._callbacks:
            try:
                try:
                    resp = self._callbacks[method](ctx, *args, **kw)
                except DjangoHttp404:
                    raise Http404
                else:
                    if not resp:
                        raise TypeError('Method `%s` does not return a response object' % self._callbacks[method])
                    if not response_representation and resp.data is not None:
                        return http_response(ctx.NotAcceptable())

                    return http_response(resp)
            except Http404:
                return http_response(ctx.NotFound())
            except Exception, ex:
                if settings.DEBUG:
                    tb = sys.exc_info()[2]
                else:
                    tb = None
                resp = responses.exception_response_factory(ctx, ex, tb)
                log.exception('Internal Server Error: %s', ctx.request.path,
                    exc_info=sys.exc_info(),
                    extra={
                        'status_code': resp.status,
                        'context': ctx,
                    }
                )
                return http_response(resp)
        else:
            return http_response(ctx.MethodNotAllowed({'error': 'Method `%s` is not registered for resource `%s`' % (
                method, self._path)}))

    def representation(self, name=DEFAULT_REPRESENTATION_KEY):
        def wrapped(func):
            self._representations[name] = func
            return func
        return wrapped

    def uri(self, context, params=None, query=None):
        assert params is None or isinstance(params, dict), "entity.uri() params should be passed as dict"

        params = params or {}

        uri = context.build_absolute_uri(self._path)
        uri = urltemplate.to_url(uri, params)

        if query:
            uri += '?'+urllib.urlencode(query)

        return uri

    def convert(self, context, obj, representation=None):
        """
        Converts model (`obj`) using specified or default `representation`
        within a `context`
        """

        if representation is None or representation==DEFAULT_REPRESENTATION_KEY:
            try:
                convert = self.representations[DEFAULT_REPRESENTATION_KEY]
            except KeyError:
                convert = lambda x, ctx: x  # pass through
        else:
            convert = self.representations[representation]
        return convert(obj, context)




"""
Restosaur - a tiny but real REST library

Author: Marcin Nowak <marcin.j.nowak@gmail.com>
"""


import resource
import responses
import filters
import decorators


def autodiscover(module_name='restapi'):
    from django.conf import settings

    try:
        from django.utils.module_loading import autodiscover_modules
    except ImportError:
        from django.utils.importlib import import_module
        from django.utils.module_loading import module_has_submodule
        autodiscover_modules = None

    if autodiscover_modules:
        autodiscover_modules(module_name)
    else:
        for app in settings.INSTALLED_APPS:
            mod = import_module(app)
            try:
                import_module('%s.%s' % (app, module_name))
            except:
                if module_has_submodule(mod, module_name):
                    raise


class API(object):
    def __init__(self, path, resources=None, middlewares=None):
        if not path.endswith('/'):
            path += '/'
        self.path = path
        self.resources = resources or []
        self.middlewares = middlewares or []

    def add_resources(self, *resources):
        self.resources += resources

    def resource(self, *args, **kw):
        obj = resource.Resource(*args, **kw)
        self.add_resources(obj)
        return obj

    def get_urls(self):
        from django.conf.urls import patterns, url, include
        from django.views.decorators.csrf import csrf_exempt
        from .dispatch import resource_dispatcher_factory
        import urltemplate

        urls = []

        for resource in self.resources:
            path = urltemplate.to_django_urlpattern(resource._path)
            if path.startswith('/'):
                path=path[1:]
            urls.append(url('^%s$' % path,
                csrf_exempt(resource_dispatcher_factory(self, resource))))

        return [url('^%s' % self.path, include(patterns('', *urls)))]

    def urlpatterns(self):
        from django.conf.urls import patterns, include
        return patterns('', (r'^', include(self.get_urls())))

    def autodiscover(self, *args, **kw):
        """
        Shortcut for `restosaur.autodiscover()`
        """
        autodiscover(*args, **kw)





import unittest
from restosaur import API


class APITestCase(unittest.TestCase):
    pass


class APIPathsTestCase(APITestCase):
    def test_appending_slash_to_api_path(self):
        api = API('foo')
        self.assertEqual(api.path, 'foo/')

    def test_not_appending_slash_to_api_path_if_exists(self):
        api = API('foo/')
        self.assertEqual(api.path, 'foo/')

    def test_that_root_url_pattern_does_not_contain_slash(self):
        api = API('foo')
        root = api.resource('/')
        urls = api.get_urls()
        root_url = urls[0].url_patterns[0]
        self.assertFalse('/' in root_url._regex)

    def test_that_typical_url_pattern_does_not_contain_prepending_slash(self):
        api = API('foo')
        bar = api.resource('bar/')
        urls = api.get_urls()
        bar_url = urls[0].url_patterns[0]
        self.assertEqual(bar_url._regex, '^bar/$')




from django.utils.http import http_date
from .utils import model_to_dict
import times


class Response(object):
    def __init__(self, context, data=None, status=200, headers=None,
            last_modified=None, extra=None, add_links=True, links_key='_links'):
        self.headers = {}
        self.headers.update(headers or {})
        self.representation = None
        self.content_type = None
        self.context = context
        self.status = status
        self.extra = extra
        self.links_key = links_key
        self.add_links = add_links
        self.data = data
        if last_modified:
            self.set_last_modified(last_modified)

    def get_converter(self, representation):
        dummy_converter = lambda x, context: x
        converter = self.context.resource.representations.get(representation)
        return converter or dummy_converter

    def set_last_modified(self, dt):
        if dt:
            self.headers['Last-Modified'] = http_date(times.to_unix(dt))
        else:
            self.headers.pop('Last-Modified', None)

    def serialize(self, data, representation):
        if data is None and not self.extra:
            return ''
        else:
            if self.status >= 200 and self.status <300:
                output = {}
                output.update(self.extra or {})
                output.update(self.context.resource.convert(self.context, data, representation))
                self._add_links(output, data, representation)
            else:
                output = data
            return output

    def _serialize_links(self, data, representation):
        links = {}
        if self.add_links and self.context.resource._links:
            for key in self.context.resource._links:
                method, linked_resource = self.context.resource._links[key]
                links[key]={
                    'uri': linked_resource.uri(self.context, params=self.context.parameters),
                    'method': method.upper(),
                    }
        return links

    def _add_links(self, resp, data, representation):
        if not self.links_key in resp:
            resp[self.links_key] = {}
        resp[self.links_key].update(
                self._serialize_links(data, representation))
        return resp


class CreatedResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(CreatedResponse, self).__init__(context, data=data, status=201,
                headers=headers)


class NoContentResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(NoContentResponse, self).__init__(context, data=data, status=204,
                headers=headers)


class SeeOtherResponse(Response):
    def __init__(self, context, uri, data=None, headers=None):
        headers = headers or {}
        headers['Location'] = uri
        super(SeeOtherResponse, self).__init__(context, data=data,
                status=303, headers=headers)


class NotModifiedResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(NotModifiedResponse, self).__init__(context, data=data, status=304,
                headers=headers)


class UnauthorizedResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(UnauthorizedResponse, self).__init__(context, data=data,
                status=401, headers=headers)


class ForbiddenResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(ForbiddenResponse, self).__init__(context, data=data, status=403,
                headers=headers)


class NotFoundResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(NotFoundResponse, self).__init__(context, data=data, status=404,
                headers=headers)


class MethodNotAllowedResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(MethodNotAllowedResponse, self).__init__(context, data=data,
                status=405, headers=headers)


class CollectionResponse(Response):
    def __init__(self, context, iterable, totalCount=None, key=None, **kwargs):
        super(CollectionResponse, self).__init__(context, data=iterable,
                **kwargs)
        self.key = key or 'items'
        self.totalCount = totalCount

    def serialize(self, iterable, representation):
        resp = {
                self.key: map(lambda x: self.context.resource.convert(self.context, x, representation), iterable),
                'totalCount': self.totalCount if self.totalCount is not None else len(iterable),
                }
        resp.update(self.extra or {})
        self._add_links(resp, iterable, representation)
        return resp


class EntityResponse(Response):
    pass


class NotAcceptableResponse(Response):
    def __init__(self, context, headers=None):
        super(NotAcceptableResponse, self).__init__(context, data=None,
                status=406, headers=headers)


class ValidationErrorResponse(Response):
    def __init__(self, context, errors, headers=None):
        resp = {
                'errors': errors,
                }
        super(ValidationErrorResponse, self).__init__(context, data=resp,
                status=422, headers=headers)


class InternalErrorResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(InternalErrorResponse, self).__init__(context, data=data,
                status=500, headers=headers)


class NotImplementedResponse(Response):
    def __init__(self, context, data=None, headers=None):
        super(NotImplementedResponse, self).__init__(context, data=data,
                status=501, headers=headers)


def exception_response_factory(context, ex, tb=None, extra=None):
    import traceback

    if isinstance(ex, NotImplementedError):
        cls = NotImplementedResponse
    else:
        cls = InternalErrorResponse

    data = {}
    data.update(extra or {})
    data.update({
        'error': unicode(ex),
        })

    if tb:
        def stack_trace(x):
            return dict(zip(['file','line','fn','source'], x))
        data['traceback'] = map(stack_trace, traceback.extract_tb(tb))

    return cls(context=context, data=data)






def model_to_dict(obj, context):
    """
    Convert django model instance to dict
    """

    data = {}

    for field in obj._meta.fields:
        field_name = field.column if field.rel else field.name
        data[field.column] = getattr(obj, field_name)

    return data




import unittest
import datetime
import json

from restosaur import API, responses
from restosaur.resource import Resource
from restosaur.dispatch import resource_dispatcher_factory

from django.test import SimpleTestCase


class ResourceTestCase(unittest.TestCase):
    def setUp(self):
        from django.test import RequestFactory

        super(ResourceTestCase, self).setUp()

        self.api = API('/')
        self.rqfactory = RequestFactory()

    def call(self, resource, method, *args, **kw):
        rq = getattr(self.rqfactory, method)(resource.path, *args, **kw)
        return resource_dispatcher_factory(self.api, resource)(rq)


class DefaultRepresentationTestCase(ResourceTestCase):
    def setUp(self):
        super(DefaultRepresentationTestCase, self).setUp()

        self.entity = self.api.resource('entity')

        @self.entity.get()
        def entity_GET(ctx):
            return ctx.Entity({'some':'test'})

    def test_successful_getting_200_status_code(self):
        resp = self.call(self.entity, 'get')
        self.assertEqual(resp.status_code, 200)

    def test_returning_valid_content_type(self):
        resp = self.call(self.entity, 'get')
        self.assertEqual(resp['Content-Type'], 'application/json')

    def test_getting_valid_entity_content(self):
        resp = self.call(self.entity, 'get')
        resp_json = json.loads(resp.content)
        self.assertTrue(resp_json['some'] == 'test')

    def test_raising_not_acceptable_for_unsupported_representation(self):
        resp = self.call(self.entity, 'get', HTTP_ACCEPT='application/vnd.not-defined+json')
        self.assertEqual(resp.status_code, 406)

    def test_raising_not_acceptable_for_unsupported_serializer(self):
        resp = self.call(self.entity, 'get', HTTP_ACCEPT='application/eggsandmeat')
        self.assertEqual(resp.status_code, 406)

    def test_returning_fallback_application_json_content_type_for_unsupported_serializer(self):
        resp = self.call(self.entity, 'get', HTTP_ACCEPT='application/eggsandmeat')
        self.assertEqual(resp['Content-Type'], 'application/json')


class SeeOtherTestCase(ResourceTestCase):
    def setUp(self):
        super(SeeOtherTestCase, self).setUp()

        self.seeother = self.api.resource('seeother')

        @self.seeother.get()
        def seeother_GET(ctx):
            return ctx.SeeOther('https://google.com')

    def test_that_seeother_accepts_any_content_type(self):
        resp = self.call(self.seeother, 'get', HTTP_ACCEPT='application/vnd.not-defined+json')
        self.assertEqual(resp.status_code, 303)

    def test_that_seeother_sends_back_location_header(self):
        resp = self.call(self.seeother, 'get')
        self.assertEqual(resp['Location'], 'https://google.com')

    def test_that_seeother_returns_no_content(self):
        resp = self.call(self.seeother, 'get')
        self.assertEqual(resp.content, '')

    def test_that_seeother_returns_application_json_content_type(self):
        resp = self.call(self.seeother, 'get')
        self.assertEqual(resp['Content-Type'], 'application/json')


class NotFoundTestCase(ResourceTestCase):
    def setUp(self):
        super(NotFoundTestCase, self).setUp()

        self.resource_exc = self.api.resource('notfound_exc')
        self.resource = self.api.resource('notfound')

        @self.resource.get()
        def notfound_GET(ctx):
            return ctx.NotFound()

        @self.resource_exc.get()
        def notfoundexc_GET(ctx):
            from django.http import Http404
            raise Http404

    def test_returning_404_code_when_handling_django_Http404_exception_and(self):
        resp = self.call(self.resource_exc, 'get')
        self.assertEqual(resp.status_code, 404)

    def test_valid_content_type_when_handling_django_Http404_exception_and(self):
        resp = self.call(self.resource_exc, 'get')
        self.assertEqual(resp['Content-Type'], 'application/json')

    def test_returning_404_code_when_returning_NotFoundResponse(self):
        resp = self.call(self.resource, 'get')
        self.assertEqual(resp.status_code, 404)

    def test_valid_content_type_when_returning_NotFoundResponse(self):
        resp = self.call(self.resource, 'get')
        self.assertEqual(resp['Content-Type'], 'application/json')


class MethodNotAllowedTestCase(ResourceTestCase):
    def setUp(self):
        super(MethodNotAllowedTestCase, self).setUp()
        self.empty_resource = self.api.resource('empty')

    def test_that_empty_resource_raises_method_not_allowed_for_GET(self):
        resp = self.call(self.empty_resource, 'get')
        self.assertEqual(resp.status_code, 405)

    def test_that_empty_resource_raises_method_not_allowed_for_POST(self):
        resp = self.call(self.empty_resource, 'post')
        self.assertEqual(resp.status_code, 405)

    def test_that_empty_resource_raises_method_not_allowed_for_PUT(self):
        resp = self.call(self.empty_resource, 'put')
        self.assertEqual(resp.status_code, 405)

    def test_that_empty_resource_raises_method_not_allowed_for_PATCH(self):
        resp = self.call(self.empty_resource, 'patch')
        self.assertEqual(resp.status_code, 405)

    def test_that_empty_resource_raises_method_not_allowed_for_DELETE(self):
        resp = self.call(self.empty_resource, 'delete')
        self.assertEqual(resp.status_code, 405)

    def test_that_empty_resource_raises_method_not_allowed_for_OPTIONS(self):
        resp = self.call(self.empty_resource, 'options')
        self.assertEqual(resp.status_code, 405)


class MethodsHandlingTestCase(ResourceTestCase):
    def setUp(self):
        super(MethodsHandlingTestCase, self).setUp()
        self.get = self.api.resource('get')
        self.post = self.api.resource('post')
        self.delete = self.api.resource('delete')
        self.patch = self.api.resource('patch')
        self.put = self.api.resource('put')
        self.options = self.api.resource('options')

        @self.post.post()
        @self.get.get()
        @self.patch.patch()
        @self.put.put()
        @self.options.options()
        def response_200_OK(ctx):
            return ctx.Response()

    def test_succesful_handling_registered_GET(self):
        resp = self.call(self.get, 'get')
        self.assertEqual(resp.status_code, 200)

    def test_succesful_handling_registered_POST(self):
        resp = self.call(self.post, 'post')
        self.assertEqual(resp.status_code, 200)

    def test_succesful_handling_registered_PUT(self):
        resp = self.call(self.put, 'put')
        self.assertEqual(resp.status_code, 200)

    def test_succesful_handling_registered_PATCH(self):
        resp = self.call(self.patch, 'patch')
        self.assertEqual(resp.status_code, 200)

    def test_succesful_handling_registered_OPTIONS(self):
        resp = self.call(self.options, 'options')
        self.assertEqual(resp.status_code, 200)

    def test_not_handling_notregistered_POST(self):
        for resource in (self.get, self.put, self.patch, self.options):
            resp = self.call(resource, 'post')
            self.assertEqual(resp.status_code, 405)

    def test_not_handling_notregistered_PUT(self):
        for resource in (self.get, self.post, self.patch, self.options):
            resp = self.call(resource, 'put')
            self.assertEqual(resp.status_code, 405)

    def test_not_handling_notregistered_GET(self):
        for resource in (self.put, self.post, self.patch, self.options):
            resp = self.call(resource, 'get')
            self.assertEqual(resp.status_code, 405)

    def test_not_handling_notregistered_PATCH(self):
        for resource in (self.put, self.post, self.get, self.options):
            resp = self.call(resource, 'patch')
            self.assertEqual(resp.status_code, 405)

    def test_not_handling_notregistered_OPTIONS(self):
        for resource in (self.put, self.post, self.get, self.patch):
            resp = self.call(resource, 'options')
        self.assertEqual(resp.status_code, 405)


class ExceptionsHandlingTestCase(ResourceTestCase, SimpleTestCase):
    def setUp(self):
        super(ExceptionsHandlingTestCase, self).setUp()

        self.exc_resource = self.api.resource('exception')
        self.notimpl_resource = self.api.resource('not-implemented')

        @self.exc_resource.get()
        def raise_some_exception(ctx):
            raise Exception('Test exception')

        @self.notimpl_resource.get()
        def raise_not_impl_exception(ctx):
            raise NotImplementedError('This code is not implemented')

    def test_successful_returning_internal_server_error_status_500(self):
        resp = self.call(self.exc_resource, 'get')
        self.assertEqual(resp.status_code, 500)

    def test_successful_returning_internal_server_error_message(self):
        resp = self.call(self.exc_resource, 'get')
        resp_json = json.loads(resp.content)
        self.assertEqual(resp_json['error'], 'Test exception')

    def test_not_returning_internal_server_error_traceback_when_debug_is_off(self):
        with self.settings(DEBUG=False):
            resp = self.call(self.exc_resource, 'get')
            resp_json = json.loads(resp.content)
            self.assertFalse('traceback' in resp_json)

    def test_successful_returning_internal_server_error_traceback_when_debug_is_on(self):
        with self.settings(DEBUG=True):
            resp = self.call(self.exc_resource, 'get')
            resp_json = json.loads(resp.content)
            self.assertTrue('traceback' in resp_json)

    def test_returning_internal_server_error_traceback_as_list(self):
        with self.settings(DEBUG=True):
            resp = self.call(self.exc_resource, 'get')
            resp_json = json.loads(resp.content)
            self.assertTrue(isinstance(resp_json['traceback'],list))

    def test_returning_valid_internal_server_error_traceback_entity(self):
        with self.settings(DEBUG=True):
            resp = self.call(self.exc_resource, 'get')
            resp_json = json.loads(resp.content)
            entity = resp_json['traceback'][0]

            self.assertTrue('source' in entity)
            self.assertTrue('line' in entity)
            self.assertTrue('fn' in entity)
            self.assertTrue('file' in entity)

    def test_successful_returning_not_implemented_error_message(self):
        resp = self.call(self.notimpl_resource, 'get')
        resp_json = json.loads(resp.content)
        self.assertEqual(resp_json['error'], 'This code is not implemented')

    def test_successful_returning_not_implemented_error_status_501(self):
        resp = self.call(self.notimpl_resource, 'get')
        self.assertEqual(resp.status_code, 501)




from .forms import modelform_factory, model_to_dict


class Representation(object):
    links = {}

    def __init__(self, context):
        self.context = context

    def to_dict(self, obj):
        data = {}
        data.update(obj)
        data.update({
            '_links': self.links,
            })
        return data


class ModelRepresentation(Representation):
    model = None
    form = None
    fields = '__all__'

    def get_form(self):
        if self.form:
            return self.form
        return modelform_factory(self.model, fields=self.fields)

    def to_dict(self, obj):
        return model_to_dict(obj)




import functools
from .responses import UnauthorizedResponse, ForbiddenResponse



def login_required(func):
    @functools.wraps(func)
    def wrapped(request, *args, **kw):
        if not request.user.is_authenticated():
            return UnauthorizedResponse(request)
        return func(request, *args, **kw)
    return wrapped


def staff_member_required(func):
    @functools.wraps(func)
    @login_required
    def wrapped(request, *args, **kw):
        if not request.user.is_staff:
            return ForbiddenResponse(request)
        return func(request, *args, **kw)
    return wrapped



from .context import Context


def querydict_to_dict(qd):
    out = {}
    for key in qd:
        if len(qd.getlist(key))>1:
            out[key]=qd.getlist(key)
        else:
            out[key]=qd.get(key)
    return out


def build_context(api, resource, request):
    try:
        raw_body = request.body # Django may raise RawPostDataException sometimes;
                                # i.e. when processing POST multipart/form-data;
                                # In that cases we can't access raw body anymore, sorry
    except:
        raw_body = None

    parameters = {}

    if request.resolver_match:
        parameters.update(request.resolver_match.kwargs)

    parameters.update(querydict_to_dict(request.GET))

    ctx = Context(api, request=request, resource=resource,
        method=request.method, parameters=parameters, data=request.POST,
        files=request.FILES, raw=raw_body)

    return ctx


def resource_dispatcher_factory(api, resource):
    def dispatch_request(request, *args, **kw):
        ctx = build_context(api, resource, request)

        for middleware in api.middlewares:
            try:
                method = middleware.process_request
            except AttributeError:
                pass
            else:
                if method(request, ctx) == False:
                    break

        response = resource(ctx, *args, **kw)

        for middleware in api.middlewares:
            try:
                method = middleware.process_response
            except AttributeError:
                pass
            else:
                if method(request, response, ctx) == False:
                    break

        return response
    return dispatch_request





from restosaur import API


class ResourceAlreadyRegistered(Exception):
    pass


class ApiRoot(object):
    def __init__(self):
        self.resources = {}

    def register(self, resource, name):
        if name in self.resources:
            raise ResourceAlreadyRegistered(name)
        self.resources[name]=resource

    def expose(self, name, resource):
        def wrap(resource):
            self.register(resource, name)
            return resource
        return wrap

    def as_view(self):
        def get_api_root(ctx):
            data = {}

            for name, resource in self.resources.items():
                data[name]=resource.uri(ctx)

            return ctx.Entity(data)
        return get_api_root





import responses
import email
import times
import urllib
import urlparse

from django.utils.encoding import force_bytes # todo: implement own conversion utility


def parse_http_date(header, headers):
    if header in headers and headers[header]:
        timetuple = email.utils.parsedate_tz(headers[header])
        try:
            return times.from_unix(email.utils.mktime_tz(timetuple))
        except (TypeError, ValueError):
            pass


class Context(object):
    def __init__(self, api, request, resource, method, parameters=None,
            body=None, data=None, files=None, raw=None, extra=None, headers=None):
        self.method = method
        self.api = api
        self.headers = headers or {}
        self.request = request
        self.body = body
        self.raw = raw
        self.resource = resource
        self.parameters = parameters or {} # GET
        self.data = data or {} # POST
        self.files = files or {} # FILES
        self.deserializer = None
        self.content_type = None
        self.extra = extra or {}

    def build_absolute_uri(self, path=None, parameters=None):
        """
        Returns absolute uri to the specified `path` with optional
        query string `parameters`.

        If no `path` is provided, the current request full path
        (including query string) will be used and extended by
        optional `parameters`.
        """

        def build_uri(path):
            current = 'http%s://%s%s' % ('s' if self.request.is_secure() else '',
                    self.request.get_host(), self.request.path)
            return urlparse.urljoin(current, path)

        params = {}
        if path:
            full_path = u'/'.join(filter(None, (self.api.path+path).split('/')))
            if path.endswith('/'):
                full_path+='/'
            uri = build_uri('/'+full_path)
        else:
            params = dict(self.parameters)
            uri = build_uri(self.request.path)

        enc = self.request.GET.encoding # todo: change to internal restosaur settings

        params.update(parameters or {})
        params = dict(map(lambda x: (force_bytes(x[0], enc), force_bytes(x[1], enc)),
            params.items()))

        if params:
            return '%s?%s' % (uri, urllib.urlencode(params))
        else:
            return uri

    def url_for(self, resource, **kwargs):
        """
        Shortcut wrapper of `resource.uri()`
        """
        return resource.uri(self, params=kwargs)

    def is_modified_since(self, dt):
        """
        Compares datetime `dt` with `If-Modified-Since` header value.
        Returns True if `dt` is newer than `If-Modified-Since`, False otherwise.
        """
        if_modified_since = parse_http_date('if-modified-since', self.headers)

        if if_modified_since:
            return times.to_unix(dt.replace(microsecond=0)) > times.to_unix(if_modified_since)

        return True

    @property
    def deserialized(self):
        return self.body

    # response factories

    def Response(self, *args, **kwargs):
        return responses.Response(self, *args, **kwargs)

    def Created(self, *args, **kwargs):
        return responses.CreatedResponse(self, *args, **kwargs)

    def ValidationError(self, *args, **kwargs):
        return responses.ValidationErrorResponse(self, *args, **kwargs)

    def NotAcceptable(self, *args, **kwargs):
        return responses.NotAcceptableResponse(self, *args, **kwargs)

    def NotFound(self, *args, **kwargs):
        return responses.NotFoundResponse(self, *args, **kwargs)

    def SeeOther(self, *args, **kwargs):
        return responses.SeeOtherResponse(self, *args, **kwargs)

    def NotModified(self, *args, **kwargs):
        return responses.NotModifiedResponse(self, *args, **kwargs)

    def MethodNotAllowed(self, *args, **kwargs):
        return responses.MethodNotAllowedResponse(self, *args, **kwargs)

    def Forbidden(self, *args, **kwargs):
        return responses.ForbiddenResponse(self, *args, **kwargs)

    def Unauthorized(self, *args, **kwargs):
        return responses.UnauthorizedResponse(self, *args, **kwargs)

    def NoContent(self, *args, **kwargs):
        return responses.NoContentResponse(self, *args, **kwargs)

    def Entity(self, *args, **kwargs):
        return responses.EntityResponse(self, *args, **kwargs)

    def Collection(self, *args, **kwargs):
        return responses.CollectionResponse(self, *args, **kwargs)

